#!/usr/bin/env python3

import sys
import os
import dnf
import subprocess
import shutil
from dnf.cli import progress

def install_package(package_names, rootpath, rpmdb_path):
    base = dnf.Base()

    conf = base.conf

    # 设置 RPM 数据库路径
    conf.installroot = rootpath.replace('"', '')
    conf.rpmdb_path = rpmdb_path

    # 禁用脚本自动执行
    conf.tsflags.append('noscripts')

    base.read_all_repos()  # 加载远程仓库
    base.fill_sack()

    # 添加要安装的包
    for package_name in package_names:
        try:
            base.install(package_name)
        except dnf.exceptions.PackageNotFoundError as e:
            print(f"找不到软件包：{package_name}")
            return []

    # 解析依赖关系
    try:
        base.resolve()
    except dnf.exceptions.DepsolveError as e:
        print(f"无法解决依赖关系：{e}")
        return []

    if base.transaction:
        # 执行安装事务
        try:
            print("正在下载软件包...")
            progress_meter = dnf.cli.progress.MultiFileProgressMeter()
            base.download_packages(base.transaction.install_set, progress_meter)
            installed_packages = [pkg for pkg in base.transaction.install_set]
            return [p.localPkg() for p in installed_packages]
            #print([p.localPkg() for p in installed_packages])
            #print("正在安装...")
            ## use the install func in rpm_operations.c
            #base.do_transaction()
            #print(f"安装成功：{' '.join(package_names)}")

            # 创建硬链接
            #installed_packages = [pkg for pkg in base.transaction.install_set]
            #for pkg in installed_packages:
            #    create_hardlinks(rootpath, pkg)

        except dnf.exceptions.Error as e:
            print(f"下载失败：{e}")
            return []
    else:
        print("软件包已安装或无需更新。")

# TODO
def remove_package(package_names, rootpath, rpmdb_path):
    base = dnf.Base()

    conf = base.conf
    conf.installroot = rootpath
    conf.rpmdb_path = rpmdb_path

    base.read_all_repos()  # 加载远程仓库，处理依赖
    base.fill_sack()

    # 添加要卸载的包
    for package_name in package_names:
        try:
            base.remove(package_name)
        except dnf.exceptions.PackagesNotInstalledError as e:
            print(f"以下软件包未安装：{package_name}")

    # 解析依赖关系
    try:
        base.resolve()
    except dnf.exceptions.DepsolveError as e:
        print(f"无法解决依赖关系：{e}")
        sys.exit(1)

    if base.transaction:
        # 执行卸载事务
        try:
            # 删除硬链接
            remove_hardlinks(base.transaction.remove_set)
            base.do_transaction()
            print(f"卸载成功：{' '.join(package_names)}")
        except dnf.exceptions.Error as e:
            print(f"卸载失败：{e}")
            sys.exit(1)
    else:
        print("软件包未安装或无需卸载。")

def query_package(package_names):
    base = dnf.Base()
    base.read_all_repos()           # 读取所有远程仓库
    base.fill_sack()

    for package_name in package_names:
        # 查询远程仓库中的软件包
        q = base.sack.query().available().filter(name=package_name)

        if q:
            for pkg in q:
                print(f"仓库中可用的软件包：{pkg.name}-{pkg.version}-{pkg.release}.{pkg.arch}")
                print(f"仓库：{pkg.reponame}")
                print(f"包大小：{pkg.size} bytes")
                print(f"摘要：{pkg.summary}")
                print(f"描述：{pkg.description}")
                print('-' * 60)
        else:
            print(f"在远程仓库中未找到软件包：{package_name}")

def create_hardlinks(rootpath, package):
    pkg_files = package.files
    for file_path in pkg_files:
        src = os.path.join(rootpath, file_path.lstrip('/'))
        dst = file_path

        # 如果目标文件已存在，跳过
        if os.path.exists(dst):
            continue

        if dst.startswith("/usr/lib/.build-id"):
            continue

        # 创建目标目录
        os.makedirs(os.path.dirname(dst), exist_ok=True)
        # 创建硬链接
        try:
            os.link(src, dst)
            print(f"创建硬链接：{dst}")
        except FileNotFoundError:
            print(f"无法创建硬链接，源文件不存在：{src}")
        except PermissionError:
            pass
            #print(f"没有权限创建硬链接：{dst}")

def remove_hardlinks(remove_set):
    for package_name in remove_set:
        pkg_files = package_name.files
        for file_path in pkg_files:
            dst = file_path
            if dst.startswith("/usr/lib/.build-id"):
                continue

            if not os.path.exists(dst):
                continue

            try:
                if os.path.isdir(dst):
                    print(f"Removing directory: {dst}")
                    shutil.rmtree(dst)  # 递归删除目录及其内容
                else:
                    print(f"Removing file: {dst}")
                    os.remove(dst)  # 删除文件
            except Exception as e:
                print(f"Error removing {dst}: {e}")

def execute_scripts(base, rootpath):
    import rpm

    ts = rpm.TransactionSet(rootpath)
    for tsi in base.transaction.trans_sequence:
        header = tsi.pkg
        pkg_name = header.name
        for script_tag in [rpm.RPMTAG_PREIN, rpm.RPMTAG_POSTIN, rpm.RPMTAG_PREUN, rpm.RPMTAG_POSTUN]:
            script = header.scripts.get(script_tag)
            if script:
                script_type = rpm.expandMacro(ts, rpm.expandTag(script_tag))
                # 保存脚本到临时文件
                script_file = f"/tmp/{pkg_name}-{script_type}.sh"
                with open(script_file, 'w') as f:
                    f.write(script)

                # 设置可执行权限
                os.chmod(script_file, 0o755)

                # 在根目录下执行脚本
                try:
                    subprocess.run([script_file], cwd='/', check=True)
                    print(f"执行脚本：{script_file}")
                except subprocess.CalledProcessError as e:
                    print(f"执行脚本失败：{script_file}")
                finally:
                    # 删除临时脚本文件
                    os.remove(script_file)

if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="DNF 包管理示例")
    parser.add_argument('action', choices=['install', 'remove', 'query'], help='操作：install，remove，query')
    parser.add_argument('packages', nargs='+', help='包名（支持多个）')

    args = parser.parse_args()

    rootpath = "/mnt"  # 指定的 rootpath
    rpmdb_path = "/mnt/usr/lib/sysimage/rpm/"  # 指定的 RPM 数据库路径

    if args.action == 'install':
        print(install_package(args.packages, rootpath, rpmdb_path))
    elif args.action == 'remove':
        remove_package(args.packages, rootpath, rpmdb_path)
    elif args.action == 'query':
        query_package(args.packages)
